/*************************************************************************
 *  Compilation:  javac PlayThatTune.java
 *  Execution:    java PlayThatTune
 *  Dependencies: StdAudio.java, GuitarString.java, Guitar.java, Guitar37.java
 *
 *  Modified 4/11/12 by Stuart Reges to use a JFileChooser instead of
 *  redirecting input from a file and to use a Scanner for input and to play
 *  the tones using a Guitar37 object (ignoring notes that it cannot play).
 *  This version brings up a file chooser that defaults to the current
 *  directory.
 *************************************************************************/

/*************************************************************************
 *  Compilation:  javac PlayThatTune.java
 *  Execution:    java PlayThatTune < input.txt
 *  Dependencies: StdAudio.java StdAudio.java
 *
 *  This is a data-driven program that plays pure tones from
 *  the notes on the chromatic scale, specified on standard input
 *  by their distance from concert A.
 *
 *  % java PlayThatTune < elise.txt
 *
 *
 *  Data files
 *  ----------
 *  http://www.cs.princeton.edu/introcs/21function/elise.txt
 *  http://www.cs.princeton.edu/introcs/21function/99luftballons.txt
 *  http://www.cs.princeton.edu/introcs/21function/freebird.txt
 *  http://www.cs.princeton.edu/introcs/21function/Ascale.txt
 *  http://www.cs.princeton.edu/introcs/21function/National_Anthem.txt
 *  http://www.cs.princeton.edu/introcs/21function/looney.txt
 *  http://www.cs.princeton.edu/introcs/21function/StairwayToHeaven.txt
 *  http://www.cs.princeton.edu/introcs/21function/entertainer.txt
 *  http://www.cs.princeton.edu/introcs/21function/old-nassau.txt
 *  http://www.cs.princeton.edu/introcs/21function/arabesque.txt
 *  http://www.cs.princeton.edu/introcs/21function/firstcut.txt 
 *  http://www.cs.princeton.edu/introcs/21function/tomsdiner.txt
 *
 *************************************************************************/

import javax.swing.*;              // for JFileChooser
import javax.swing.filechooser.*;  // for FileNameExtensionFilter
import java.io.*;                  // for FileNotFoundException
import java.util.*;                // for Scanner

public class PlayThatTune {
    public static final String KEYBOARD =
        "q2we4r5ty7u8i9op-[=zxdcfvgbnjmk,.;/' ";  // keyboard layout

    public static void main(String[] args) throws FileNotFoundException {

        JFileChooser chooser = new JFileChooser(new File("."));
        FileNameExtensionFilter filter =
            new FileNameExtensionFilter("music files", "txt");
        chooser.setFileFilter(filter);
        int result = chooser.showOpenDialog(null);
        if (result != JFileChooser.APPROVE_OPTION)
            return;
        Scanner input = new Scanner(chooser.getSelectedFile());

        Guitar g = new Guitar37();
        int time = 0;

        // repeat as long as there are more integers to read in
        while (input.hasNextInt()) {

            // read in the pitch, where 0 = Concert A (A4)
            int pitch = input.nextInt();
            // convert the pitch to the appropriate guitar string index
            int index = pitch + 12;
            // if the note is included in our keyboard, play it
            if (index >= 0 && index < KEYBOARD.length()) {
                g.pluck(KEYBOARD.charAt(index));
            }

            // read in duration in seconds and convert to a tic count
            double duration = input.nextDouble();
            int target = time + 
                (int) Math.round(duration * StdAudio.SAMPLE_RATE);

            // keep playing the guitar for that duration
            while (time < target) {
                g.play();
                g.tic();
                time++;
            }
        }
    }
}
